/* dndlist.c : List potential targets of dropped objects.
 *
 * Version 1.1
 *
 * Copyright (c) 2004 by Callum McKenzie.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */

/*
 * Version 1.1: Lars Wirzenius added "Clear" button and made window size bigger.
 */


#include <gtk/gtk.h>

/* Ensure we only report the data once. */
gboolean item_handled = FALSE;

GtkTextBuffer * textbuffer;

void clear_cb (GtkWidget *widget, gpointer data)
{
  item_handled = FALSE;
  gtk_text_buffer_set_text(textbuffer, "", 0);
}

void quit_cb (GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
}

gboolean drag_motion_cb (GtkWidget *widget, GdkDragContext *context, gint x, 
			 gint y, guint time, gpointer user)
{
  GList * targets;
  gchar * string;
  gboolean add_nl = FALSE;

  if (item_handled)
    return TRUE;

  gtk_text_buffer_set_text (textbuffer, "", 0);

  targets = context->targets;
  while (targets) {
    if (add_nl)
      gtk_text_buffer_insert_at_cursor (textbuffer, "\n", 1);
    add_nl = TRUE;
    string = gdk_atom_name (GDK_POINTER_TO_ATOM (targets->data));
    gtk_text_buffer_insert_at_cursor (textbuffer, string, -1);
    targets = g_list_next (targets);
  }

  gdk_drag_status (context, GDK_ACTION_PRIVATE, time);

  item_handled = TRUE;

  return TRUE;
}

void drag_leave_cb (GtkWidget *widget, GdkDragContext *context, guint time, 
		    gpointer user)
{
  item_handled = FALSE;
}

gboolean keypress_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  GtkTextIter start;
  GtkTextIter end;
  GtkClipboard * primary = NULL;
  GtkClipboard * clipboard = NULL;

  if (primary == NULL) {
    primary = gtk_clipboard_get (gdk_atom_intern ("PRIMARY", FALSE));
    clipboard = gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", FALSE));
  }

  if (event->state == GDK_CONTROL_MASK) {
    switch (event->keyval) {
    case 'q':
      quit_cb (NULL, NULL);
      break;
    case 'a':
      gtk_text_buffer_get_bounds (textbuffer, &start, &end);
      gtk_text_buffer_select_range (textbuffer, &start, &end);
      gtk_text_buffer_copy_clipboard (textbuffer, primary);
      break;
    case 'c':
      gtk_text_buffer_copy_clipboard (textbuffer, clipboard);
      break;
    }
  }

  return TRUE;
}

int main (int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *label;
  GtkWidget *vbox;
  GtkWidget *textview;
  GtkWidget *frame;
  GtkWidget *buttonbox;
  GtkWidget *button;

  gtk_init (&argc, &argv);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_drag_dest_set (window, 0, NULL, 0, GDK_ACTION_PRIVATE);
  gtk_widget_add_events (window, GDK_KEY_PRESS_MASK);
  g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK (quit_cb),
		    NULL);
  g_signal_connect (G_OBJECT (window), "drag-motion", 
		    G_CALLBACK (drag_motion_cb), NULL);
  g_signal_connect (G_OBJECT (window), "drag-leave", 
		    G_CALLBACK (drag_leave_cb), NULL);
  g_signal_connect (G_OBJECT (window), "key-press-event", 
		    G_CALLBACK (keypress_cb), NULL);
  gtk_window_set_default_size (GTK_WINDOW (window), 128, 256);
  gtk_window_set_title (GTK_WINDOW (window), "DND List");

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  label = gtk_label_new ("Drag objects over this window to get a list of targets");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);

  frame = gtk_frame_new ("Types");
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 5);

  textbuffer = gtk_text_buffer_new (NULL);
  textview = gtk_text_view_new_with_buffer (textbuffer);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
  gtk_container_add (GTK_CONTAINER (frame), textview);

  buttonbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (buttonbox), GTK_BUTTONBOX_END);
  gtk_box_pack_start (GTK_BOX (vbox), buttonbox, FALSE, FALSE, 5);
  gtk_box_set_spacing (GTK_BOX (buttonbox), 5);
  
  button = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (clear_cb), NULL);
  gtk_box_pack_start_defaults (GTK_BOX (buttonbox), button);

  button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (quit_cb), NULL);
  gtk_box_pack_start_defaults (GTK_BOX (buttonbox), button);

  gtk_widget_show_all (window);

  gtk_main ();

  return 0;
}
